package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class Synthetic extends Attribute
{
    private byte[] bytes;
    
    public Synthetic(final int nameIndex, final int length, final byte[] bytes, final ConstantPool constantPool) {
        super((byte)7, nameIndex, length, constantPool);
        this.bytes = bytes;
    }
    
    Synthetic(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (byte[])null, constantPool);
        if (length > 0) {
            input.readFully(this.bytes = new byte[length]);
            Attribute.println("Synthetic attribute with length > 0");
        }
    }
    
    public Synthetic(final Synthetic c) {
        this(c.getNameIndex(), c.getLength(), c.getBytes(), c.getConstantPool());
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitSynthetic(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final Synthetic c = (Synthetic)this.clone();
        if (this.bytes != null) {
            c.bytes = this.bytes.clone();
        }
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        if (super.getLength() > 0) {
            file.write(this.bytes, 0, super.getLength());
        }
    }
    
    public byte[] getBytes() {
        return this.bytes;
    }
    
    public void setBytes(final byte[] bytes) {
        this.bytes = bytes;
    }
    
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder("Synthetic");
        if (super.getLength() > 0) {
            buf.append(" ").append(Utility.toHexString(this.bytes));
        }
        return buf.toString();
    }
}
